from __future__ import division

import pytest
import sys

import limix_ext as lxt

from numpy import array, dot, empty, hstack, ones, pi, sqrt, zeros
from numpy import asarray
from numpy.random import RandomState
from numpy.testing import assert_almost_equal

@pytest.mark.skipif('linux' not in sys.platform,
                    reason="requires Linux")
def test_macau():


    random = RandomState(139)
    nsamples = 1000
    nfeatures = 1500

    G = random.randn(nsamples, nfeatures) / sqrt(nfeatures)

    u = random.randn(nfeatures)

    z = 0.1 + 2 * dot(G, u) + random.randn(nsamples)

    ntrials = random.randint(10, 500, size=nsamples)

    y = zeros(nsamples)
    for i in range(len(ntrials)):
        y[i] = sum(z[i] + random.logistic(scale=pi / sqrt(3),
                                          size=ntrials[i]) > 0)
    M = ones((nsamples, 1))

    K = G.dot(G.T)
    (stats, pvals) = lxt.macau.scan(y, ntrials, M, G[:,0:5], K)

    pvals_expected = array([0.02004, 0.4066, 0.5314, 0.9621, 0.8318])
    stats_expected = array([0, 0, 0, 0, 0])

    pvals = asarray(pvals)
    stats = asarray(stats)

    assert_almost_equal(pvals/10, pvals_expected/10, decimal=1)
    assert_almost_equal(stats/10, stats_expected/10, decimal=1)



from __future__ import absolute_import

from numpy import asarray

import numpy as np

import logging

from .core import run_scan

from ..util import gower_normalization

def scan(y, ntrials, covariate, X, K):
    logger = logging.getLogger(__name__)
    logger.info('Gower normalizing')
    K = gower_normalization(asarray(K, float))
    X = asarray(X, float).copy()
    y = asarray(y, float).copy()
    covariate = asarray(covariate, float).copy()
    ntrials = asarray(ntrials, float).copy()

    ok = X.std(0) > 0
    pvals = np.ones(X.shape[1])
    stats = np.zeros(X.shape[1])
    logger.info('macau scan started')


    pvals_ = run_scan(y, ntrials, K, X[:,ok])
    logger.info('macau scan finished')
    pvals[ok] = np.asarray(pvals_, float).ravel()

    return (stats, pvals)



def gower_normalization(K):
    """Perform Gower normalizion on covariance matrix K.

    The rescaled covariance matrix has sample variance of 1.
    """
    c = (K.shape[0]-1) / (K.trace() - K.mean(0).sum())
    return c * K



import os
import logging
import numpy as np
import subprocess
from pandas import DataFrame
import tempfile
import pandas as pd

def _run_scan(y_fp, ntrials_fp, K_fp, predictor_fp):
    outfolder = tempfile.mkdtemp()
    here = os.path.abspath(os.path.dirname(__file__))
    cmd = [os.path.join(here, 'macau'), '-g', y_fp, '-t', ntrials_fp,
                  '-p', predictor_fp,
                  '-k', K_fp, '-o', outfolder]

    os.makedirs(os.path.join(outfolder, 'output', 'tmp'))
    logger = logging.getLogger(__name__)
    msg = "Running shell list %s.", str(cmd)
    logger.debug(msg)
    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                         cwd=outfolder)
    out, err = p.communicate()
    logger.debug(out)
    if len(err) > 0:
        logger.warn(err)

    assoc_file = os.path.join(outfolder, 'output', 'tmp',
                              os.path.basename(outfolder) + '.assoc.txt')
    df = pd.read_csv(assoc_file, sep='\t', index_col=0)

    return df

def run_scan(y, ntrials, K, X):
    outfolder = tempfile.mkdtemp()

    np.savetxt(os.path.join(outfolder, 'K.txt'), K, fmt='%.10f')

    y_df = DataFrame()


    ntrials_df = DataFrame()

    for i in range(K.shape[0]):
        y_df['sample_%d' % i] = [int(y[i])]
        ntrials_df['sample_%d' % i] = [int(ntrials[i])]

    y_df.index = ['site1']
    y_df.index.name = 'site'

    ntrials_df.index = ['site1']
    ntrials_df.index.name = 'site'

    y_df.to_csv(os.path.join(outfolder, 'y.txt'), sep=" ")
    ntrials_df.to_csv(os.path.join(outfolder, 'ntrials.txt'), sep=" ")

    dfs = DataFrame()
    for i in range(X.shape[1]):
        np.savetxt(os.path.join(outfolder, 'predictor.txt'), X[:,i],
                   fmt='%.2f')

        df = _run_scan(os.path.join(outfolder, 'y.txt'),
                  os.path.join(outfolder, 'ntrials.txt'),
                  os.path.join(outfolder, 'K.txt'),
                  os.path.join(outfolder, 'predictor.txt'))
        df.index = [i]
        dfs = dfs.append(df)

    df.index.name = 'snp'
    return np.asarray(dfs['pvalue'])
